/*
 * Project name:
     Hall current click (Using mikroE's Current click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release (FJ);
 * Description:
     This project is a simple demonstration of working with the Hall Current Click board
     based on the TLI4970 high precision miniature coreless magnetic current sensorwith SPI Interface.
     This example demosntrates measuring the current drawn by the consumer.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7 Connectivity
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8MHz Crystal, 32MHz MCU Clock
     ext. modules:    hall current click board click
                      http://www.mikroe.com/click/hall-current/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Place Hall current click board in the mikroBUS socket 1.
     - Turn on LCD backlight on SW4.6.
     - Maximal SPI clock frequency for TLI4970 running at 3.3V is 5MHz.
 */

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

sbit Chip_Select           at RE0_bit;
sbit Chip_Select_Direction at TRISE0_bit;

char txt[15];
int current_read;
float current_read_float;

// Get current value
unsigned Hall_Read() {
  // Local variable
  unsigned int tmp = 0, tmp0 = 0, tmp1 = 0;
  unsigned int buffer = 0;

  Chip_Select = 0;          // Select MCP3201
  tmp0 = SPI1_Read(buffer); // Get value
  tmp1 = SPI1_Read(buffer); // Get value
  Chip_Select = 1;          // Deselect MCP3201
  
  tmp = ((tmp1) | (tmp0 << 8));

  return tmp;               // Returns value
}

void main() {
  ANSELC = 0;
  ANSELE = 0;
  
  // Set chip select pin to be output
  Chip_Select_Direction = 0;
  Chip_Select = 1;                                              // Deselect MCP3201

  // Initialize SPI1 module
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV16, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_HIGH_2_LOW);
  Delay_ms(100);

  // LCD Init
  LCD_Init();
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  
  // Display string on the LCD
  Lcd_Out(1,1,"Current [A] :");

  while(1) {
    // Read current from Hall Click
    current_read = Hall_Read();
    
    // If Sensor Current Value Transmission
    if ((current_read >> 15) == 0) {
      current_read = current_read & 0x1FFF;                     // Get [13:0] bits of current value
      current_read = current_read - 4096;                       // Map the current value
      current_read_float = (float)current_read * 0.0125;        // Calculate the current value in Amperes
      FloatToStr(current_read_float,txt);                       // Convert the current value to a string
      sprintf(txt, "%2.2f", current_read_float);                // Trim the string from leading spaces

      // Display the current value
      Lcd_Out(2,1,txt);
    }
    delay_ms(1000);
  }
}